#!/usr/bin/env python3

import itertools
import math
import os
import random
import shutil


def grouper(iterable, n):
    "Collect data into fixed-length chunks or blocks"
    args = [iter(iterable)] * n
    groups = itertools.zip_longest(*args, fillvalue=None)
    return ((x for x in g if x is not None) for g in groups)


def main(source_dir, dest_prefix, split_quant, randomize):
    """Main entry point."""
    # get source files
    source_files = os.listdir(source_dir)
    if randomize:
        random.shuffle(source_files)
    else:
        source_files.sort()

    pads = math.ceil(math.log2(split_quant))
    groups_len = math.ceil(len(source_files) / split_quant)
    for i, groups in enumerate(grouper(source_files, groups_len)):
        dest_dir = "{:s}{:0{pads}d}".format(dest_prefix, i, pads=pads)
        print("Copying to", dest_dir)
        if os.path.exists(dest_dir):
            print("ERROR: destination dir already there:", repr(dest_dir))
            exit()
        os.mkdir(dest_dir)
        for fname in groups:
            src = os.path.join(source_dir, fname)
            dst = os.path.join(dest_dir, fname)
            shutil.copy(src, dst)

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(
        description="Splits a dir in several others.")
    parser.add_argument("source_dir", help="the directory to use as a source")
    parser.add_argument("dest_prefix",
                        help="the prefix for the destination directories")
    parser.add_argument("split_quant", type=int,
                        help="how many resulting dirs to have")
    parser.add_argument("--randomize", action="store_true",
                        help="if input files should be randomized between the "
                             "destinations")
    args = parser.parse_args()

    if not os.access(args.source_dir, os.R_OK):
        print("Don't see the source directory!")
        exit()
    if args.split_quant < 2:
        print("Need to split in at least 2 parts!")
        exit()

    main(args.source_dir, args.dest_prefix, args.split_quant, args.randomize)
